""" 
This class creates and laysout a page of HK parameters to be displayed in a tab.
"""

import sys
import re

#from functools import reduce

from PyQt5 import QtWidgets, QtCore, QtGui      #, uic, QtQml, QtQuick

from PyQt5.QtGui import QPainter, QBrush, QPen
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QColor

from random import randint
from collections import deque

import GroupFrame
import iws_defs

"""
[{'row': 0, 'column': 0, 'labels': [{'mib_name': 'VIS_CDPU_5V_CPU', 'mib_id': 'VDGT2712', 'source': 'rt_slow', 'unit': 'V'}, {
"""

fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')

class GroupsFrame(QtWidgets.QFrame):

    def __init__(self, json_data, tab_index, parent, ui):
        super(GroupsFrame, self).__init__()

        self.parent = parent       # This is the tabWidget
        self.tab_index = tab_index
        self.ui = ui

        self.group_name = ""
        self.groups = []

        self.all_labels = {}
        self.all_props = {}
        self.tab_colour = 0

        check_box_frame = QtWidgets.QFrame()
        #check_box_frame.setStyleSheet("background-color:lightblue")

        font = self.font()
        font.setBold(False)
        self.setFont(font)

        self.checkbox = QtWidgets.QCheckBox("Engineering")
        self.checkbox.setObjectName("checkbox")
        self.checkbox.setChecked(True)
        self.checkbox.toggled.connect(self.engineering_check_box_clicked)
        self.checkbox.setDisabled(True)

        self.mib_checkbox = QtWidgets.QCheckBox("Use MIB name")
        self.mib_checkbox.setObjectName("mib_checkbox")
        self.mib_checkbox.setChecked(True)
        self.mib_checkbox.toggled.connect(self.mib_check_box_clicked)

        check_box_layout = QtWidgets.QHBoxLayout()
        check_box_layout.setContentsMargins(0, 0, 0, 0)
        check_box_layout.addWidget(self.checkbox)
        check_box_layout.addWidget(self.mib_checkbox, stretch = 1)
        check_box_frame.setLayout(check_box_layout)


        groups_frame = QtWidgets.QFrame()
        #groups_frame.setStyleSheet("background-color:lightgreen")

        groups_frame_layout = QtWidgets.QGridLayout()
        groups_frame_layout.setSpacing(3)
        ####groups_frame_layout.setContentsMargins(0, 0, 0, 0)
        groups_frame_layout.setContentsMargins(5, 5, 5, 5)
        
        """
        json_data = [{'row': 0, 'column': 0, 'labels': [{'mib_name': 'VIS_ROE_VCCD_01', 'mib_id': 'VS1T2038', 'source': 'rt_fast', 'unit': 'V'}, {
        """

        for group in json_data:
            row = group['row']
            col = group['column']
            labels = group['labels']
##            print(f"Length of labels {labels} = {len(labels)}")
            group = GroupFrame.GroupFrame(labels)
            groups_frame_layout.addWidget(group, row, col)

            self.groups.append(group)
            self.all_labels.update(group.labels)
            label_dict = {k:self for k in group.labels}
#            print(f'label_dict = {label_dict}')
            self.ui.all_labels.update(label_dict)
            temp_dict = { k:0 for k in group.labels }
            self.all_props.update(temp_dict)
###            print(f"GroupsFrame all_props = {self.all_props}")
##            print(f"{row},{col} : {labels}")

        groups_frame.setLayout(groups_frame_layout)


        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(check_box_frame)
        #layout.addWidget(groups_frame, stretch = 1)
        layout.addWidget(groups_frame)

        self.setLayout(layout)

        #self.debug()


    def delete_groups(self):
        self.groups.clear()


    def debug(self):
        print("Tab " + self.group_name)
        for group in self.groups:
            group.debug()
        print('all_labels:')
        print(f'{self.all_labels}')


    def parameters(self):
        dict = {}
        for group in self.groups:
            dict.update(group.parameters())
        return dict


    def engineering_check_box_clicked(self):
        if self.checkbox.isChecked():
            for group in self.groups:
                group.display_engineering()
        else:
            for group in self.groups:
                group.display_raw()


    def mib_check_box_clicked(self):
        if self.mib_checkbox.isChecked():
            for group in self.groups:
                group.display_mib_name()
        else:
            for group in self.groups:
                group.display_mib_id()


#TMPAR,\N, VMGT3017 ,VIS_PMCU_V_WINDA,\N,0.046398,\N,OK,\N,\N,3,25,4.0, 38,2023-06-23T08:01:19.654169,2023-06-23T08:01:17.781203,2023-06-23T08:01:17.781203

#GroupsFrame: update_tm: ('2023-06-23T08:01:17.781203', 'VMGT3017', '38', '0.046398', 'OK', '\\N')
#                                                              2     13      5          7     6

#TMPAR,\N,VOGT2856,VIS_OBS-G_SW_MODE,\N,\N,STANDBY,OK,\N,\N,3,25,4.0,\N,2023-06-23T08:01:21.341318,2023-06-23T08:01:19.781337,2023-06-23T08:01:19.781337
#                                                              1       2     3      4
#GroupsFrame: update_tm: ('2023-06-23T08:01:19.781337', 'VOGT2856', '\\N', '\\N', 'OK', 'STANDBY')
#                                                              2      13     5      7    6



    # fields:                                  
    # 'logTime', 'mib_id', 'rawVal', 'engVal', 'monState'
    def update_tm(self, parameter_string):
        #print(f'GroupsFrame: update_tm: {parameter_string}')

        # TESTING! TESTING! TESTING!
        #if self.tab_index == 1:
            #self.parent.tabBar().setTabTextColor(self.tab_index, QColor('red'))

###        colour_ret = 0
#        for group in self.groups:
#            ret = ret | group.update_tm(parameter_string)

#TM_MIB_ID_INDEX  = 1
#TM_RAW_VAL_INDEX = 2
#TM_ENG_VAL_INDEX = 3
#TM_STATUS_INDEX  = 4

        try:
            #mib_id = parameter_string[1]
            mib_id = parameter_string[iws_defs.TM_MIB_ID_INDEX]
            frame = self.all_labels[mib_id]

            # EDDS_FINAL
            colour_ret = frame.update_display2(parameter_string[iws_defs.TM_RAW_VAL_INDEX], \
                                               parameter_string[iws_defs.TM_ENG_VAL_INDEX], \
                                               parameter_string[iws_defs.TM_STATUS_INDEX],  \
                                               parameter_string[iws_defs.TM_ENG_VAL_INDEX + 2])
            '''
            #raw_val_str = parameter_string[2]
            #eng_val_str = parameter_string[3]
            #status      = parameter_string[4]
            raw_val_str = parameter_string[iws_defs.TM_RAW_VAL_INDEX]

            if ((frame.unit_string == 'e') and iws_defs.alter):
                eng_val_str = parameter_string[iws_defs.TM_ENG_VAL_INDEX + 2]
                raw_val = raw_val_str = '0'
               #print(f'GroupsFrame: altering {mib_id} to {eng_val_str}')
            else:
                eng_val_str = parameter_string[iws_defs.TM_ENG_VAL_INDEX]
                if fpn.match(eng_val_str):
                    eng_val = '{0:.3f}'.format(float(eng_val_str))
                    #eng_val = float("%3.03f" % float(eng_val_str))
                    eng_val_str = str(eng_val)
                else:
                    eng_val = raw_val

            status      = parameter_string[iws_defs.TM_STATUS_INDEX]

            #if fpn.match(raw_val_str):
            raw_val = float(raw_val_str)
            #else:
                #raw_val = raw_val_str

            '''
            '''
            if fpn.match(eng_val_str):
                eng_val = '{0:.3f}'.format(float(eng_val_str))
                #eng_val = float("%3.03f" % float(eng_val_str))
                eng_val_str = str(eng_val)
            else:
                eng_val = raw_val
            '''
            '''
            colour_ret = frame.update_display1(raw_val, eng_val, raw_val_str, eng_val_str, status)
            '''

            self.all_props[mib_id] = colour_ret
            self.tab_colour = self.tab_colour or colour_ret
            ret = 1

            if colour_ret == 2:
                colour = QColor('red')
                self.parent.tabBar().setTabTextColor(self.tab_index, colour)
            elif colour_ret == 3:
                colour = QColor('red')
                self.parent.tabBar().setTabTextColor(self.tab_index, colour)

        except Exception as e:
            #print(f'ex eng_val = {eng_val}')
            if mib_id == 'VOGT2856':
                print(f"GroupsFrame {self.group_name} Exception:" + str(e))
            ret = 0
            pass

###        colour = QColor('green')
        #print(f"tab {self.tab_index} ret = {ret}")

###        if colour_ret == 1:
###            colour = QColor('red')
###            print("Colour = red (1)")
###        elif colour_ret == 3:
###            colour = QColor('red')
###            print("Colour = red (3)")
###        else:
###            colour = QColor('green')
        """
        if colour_ret == 2:
            colour = QColor('yellow')
        elif ((colour_ret == 1) or (colour_ret == 3)):
            colour = QColor('red')
        else:
            colour = QColor('green')
        """

###        self.parent.tabBar().setTabTextColor(self.tab_index, colour)

        return ret


    def update_tab_colour(self):

#        res = reduce(max, list(self.all_props.values()))
        res = max(self.all_props.values())

        if res == 3:
            colour = QColor('red')
        elif res == 2:
            colour = QColor('red')
        elif res == 1:
            colour = QColor('yellow')
        else:
            colour = QColor('green')

        self.parent.tabBar().setTabTextColor(self.tab_index, colour)

        self.tab_colour = 0
