""" 
This class creates and laysout a page of HK parameters and associated plot to be displayed in a tab.
"""

import platform
import sys
import re

from PyQt5 import QtWidgets, QtCore, QtGui      #, uic, QtQml, QtQuick

system = platform.system()
if system == 'Darwin':
    # On Mac only
    from PyQt5.QtWidgets import QLayout
    from PyQt5.QtGui import QPainter, QBrush, QPen
else:
    from PyQt5.QtGui import QPainter, QBrush, QPen, QLayout

#from PyQt5.QtGui import QPainter, QBrush, QPen, QLayout
from PyQt5.QtCore import Qt

from random import randint
from collections import deque

import iws_defs

PARAMETER_LABEL_WIDTH  = 150
VALUE_LABEL_WIDTH      = 78

fpn = re.compile('[+-]?(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?')

class LabelFrame(QtWidgets.QFrame):

    """
    json_data: {'mib_name': 'VIS_ROE_CCD1_TTO01', 'mib_id': 'VS1T2020', 'source': 'rt_slow', 'unit': 'C'}
    """

    def real_label(self, json_data, label, value):
        self.name = json_data['mib_name']
        self.id   = json_data['mib_id']
        self.unit_string = json_data['unit']

        #if self.unit_string == 'e':
            #print(f'{self.name} has unit e')

        #if self.unit_string == '-':
            #self.unit_string = ' '

        #self.raw_string = "RAW"
        #self.eng_string = "ENG"
        self.raw_string = ""
        self.eng_string = ""
        self.raw_val = 0
        self.eng_val = 0.0
        self.property = 'ok'
        self.display_raw_value = 0
        label.setProperty('stat', 'ok')
        label.setText(self.name)
        label.setToolTip(self.id)
        self.label = label
        value.setProperty('stat', 'ok')
        value.setText(self.eng_string)

        self.value = value
#        if self.id == "VOGT2856":
#            print(f"{self.id} {self.name} {self.unit_string}")


    def fake_label(self, label, value):
        self.name = ""
        self.id   = ""
        self.unit_string = ""
        self.raw_string = ""
        self.eng_string = ""
        self.raw_val = 0
        self.eng_val = 0.0
        self.property = 'ok'
        self.display_raw_value = 0
        label.setText(self.name)
        label.setToolTip(self.id)
        self.label = label
        value.setText(self.eng_string)

        self.value = value


    #def __init__(self, name):
    def __init__(self, json_data):
        super(LabelFrame, self).__init__()

        style_sheet = \
            "QLabel[stat=warn] {background-color: yellow; color: black;} " +  \
            "QLabel[stat=ok] {background-color: green; color: white;} " + \
            "QLabel[stat=ool] {background-color: red; color: white}"

        label = QtWidgets.QLabel()
        label.setContentsMargins(3, 2, 0, 2)   # left, top, right, bottom
        label.setFixedWidth(PARAMETER_LABEL_WIDTH)
        label.setStyleSheet(style_sheet)

        value = QtWidgets.QLabel()
        value.setContentsMargins(0, 2, 5, 2)
        value.setFixedWidth(VALUE_LABEL_WIDTH)
        value.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        value.setStyleSheet(style_sheet)

        font = QtGui.QFont()
        font.setPointSize(6)
        #font.setPointSize(9)
        font.setBold(True)
        font.setWeight(75)
        label.setFont(font)
        value.setFont(font)

        if json_data == "":
#            print(f"LabelFrame : json_data is empty")
            self.fake_label(label, value)
        else:
            self.real_label(json_data, label, value)
#            self.name = json_data['mib_name']
#            self.id   = json_data['mib_id']
#            self.unit_string = json_data['unit']
#            if self.unit_string == '-':
#                self.unit_string = ' '
#            self.raw_string = "RAW"
#            self.eng_string = "ENG"
#            self.raw_val = 0
#            self.eng_val = 0.0
#            self.property = 'ok'
#            self.display_raw_value = 0


        
#        if json_data != "":
#            label.setProperty('stat', 'ok')
##        label.setProperty('stat', 'ok')
#        label.setText(self.name)
#        label.setToolTip(self.id)
#        self.label = label


        # Set to raw for testing only
#        #value.setProperty('stat', 'warn')
#        #value.setText(self.raw_string)
#        value.setProperty('stat', 'ok')
#        value.setText(self.eng_string)

#        self.value = value

#        font = QtGui.QFont()
#        font.setPointSize(6)
#        #font.setPointSize(9)
#        font.setBold(True)
#        font.setWeight(75)
#        label.setFont(font)
#        value.setFont(font)

        layout = QtWidgets.QHBoxLayout()
        layout.setSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(label, stretch = 1)
        layout.addWidget(value, stretch = 0)

        layout.setSizeConstraint(QLayout.SetFixedSize)

        self.setLayout(layout)

        #self.debug()


    def debug(self):
        print(self.id, self.name, self.unit_string)


    def get_total_width(self):
        return PARAMETER_LABEL_WIDTH + VALUE_LABEL_WIDTH


    def get_label_frame(self):
        return (self.id, self.name, self.unit_string)


    def display_name(self):
        self.label.setText(self.name)


    def display_mib_id(self):
        self.label.setText(self.id)

        
    def display_engineering(self):
        self.display_raw_value = 0
        self.value.setText(self.eng_string)


    def display_raw(self):
        self.display_raw_value = 1
        self.value.setText(self.raw_string)


    def update_display(self):
        self.label.setProperty('stat', self.property)
        self.value.setProperty('stat', self.property)
        self.label.setStyle(self.label.style())
        self.value.setStyle(self.value.style())
        if self.display_raw:
            self.value.setText(self.raw_string)
        else:
            self.value.setText(self.eng_string)

    
    def update_display1(self, raw_val, eng_val, raw_val_str, eng_val_str, status, display = True):
        #print(f'LabelFrame: update_display1: {self.id}, {raw_val}, {eng_val}, {raw_val_str}, {eng_val_str}, {status}')
        ret = 0
        self.raw_val = raw_val
        self.eng_val = eng_val

        if display == False:
            #print(f'LabelFrame update)display1: {self.id} not displaying {self.raw_val}, {self.eng_val}')              
            return ret

        self.raw_string = raw_val_str
        if self.unit_string in ['mV', 'V', 'mA', 'A', 'C', 'K']:
            self.eng_string = eng_val_str + " " + self.unit_string
        else:
            self.eng_string = eng_val_str + " "

        # TODO: update parameters with units '-', 'enum' in json file and test here before outputting
        if self.display_raw_value:
            self.value.setText(self.raw_string)
        else:
            if self.unit_string == '-':
                self.value.setText(self.raw_string)
            elif self.unit_string == 'e':
                #print(f'{self.name} updating with raw {self.raw_string}, eng {self.eng_string}')
                #self.value.setText(self.eng_string)
                self.value.setText(self.eng_val_str)
            else:
                self.value.setText(self.eng_string)

        if status == "ALARM":
            #print(f"{self.name} ALARM!!!")
            new_property = 'ool'
            ret = 2
            #raise Exception("ALARM")
        elif ((eng_val_str == "WARN") or (eng_val_str == "*NO VAL*")):
            new_property = 'warn'
            ret = 1
        else:
            new_property = 'ok'
            ret = 0

        self.property = new_property
        self.label.setProperty('stat', self.property)
        self.value.setProperty('stat', self.property)

        self.label.setStyle(self.label.style())
        self.value.setStyle(self.value.style())

        return ret


    #LabelFrame: update_display2:  4, 0.001276, OK, \N
    #LabelFrame: update_display2: \N, \N,       OK, IDLE
    def update_display2(self, raw_val_str, eng_val_str, status, eng_val_str1, display = True):
        #print(f'LabelFrame: update_display2: {self.id}, {raw_val_str}, {eng_val_str}, {status}, {eng_val_str1}')
        ret = 0
        #self.raw_val = raw_val
        #self.eng_val = eng_val

        #if display == False:
            #return ret


        if self.unit_string in ['mV', 'V', 'mA', 'A', 'C', 'K']:
            #self.eng_string = eng_val_str + " " + self.unit_string
            self.raw_val    = float(raw_val_str)
            self.raw_string = raw_val_str
            if fpn.match(eng_val_str):
                eng_val = '{0:.3f}'.format(float(eng_val_str))
                self.eng_val = float(eng_val)
                self.eng_val_str = str(f'{self.eng_val} {self.unit_string}')
            else:
                self.eng_val = float(raw_val_str)
                self.eng_val_str = eng_val_str
        #else:
            #self.eng_string = eng_val_str + " "
        
        elif self.unit_string == '-':
            self.raw_val     = int(raw_val_str)
            self.raw_val_str = raw_val_str
            self.eng_val     = self.raw_val
            self.eng_val_str = self.raw_val_str
            #self.eng_val_str = 'FOOO'

        else:
        #if self.unit_string == 'e':
            self.raw_val     = 0
            self.raw_val_str = '0'
            self.eng_val     = self.raw_val
            self.eng_val_str = eng_val_str1

        if display == False:
            #print(f'LabelFrame update)display1: {self.id} not displaying {self.raw_val}, {self.eng_val}')
            return ret

        self.value.setText(self.eng_val_str)

        '''
        # TODO: update parameters with units '-', 'enum' in json file and test here before outputting
        if self.display_raw_value:
            self.value.setText(self.raw_string)
        else:
            if self.unit_string == '-':
                self.value.setText(self.raw_string)
            elif self.unit_string == 'e':
                #print(f'{self.name} updating with raw {self.raw_string}, eng {self.eng_string}')
                #self.value.setText(self.eng_string)
                self.value.setText(self.eng_val_str)
            else:
                self.value.setText(self.eng_string)
        '''

        if status == "ALARM":
            #print(f"{self.name} ALARM!!!")
            new_property = 'ool'
            ret = 2
            #raise Exception("ALARM")
        elif ((status == "WARN") or (self.eng_val_str == "*NO VAL*")):
            new_property = 'warn'
            ret = 1
        else:
            new_property = 'ok'
            ret = 0

        self.property = new_property
        self.label.setProperty('stat', self.property)
        self.value.setProperty('stat', self.property)

        self.label.setStyle(self.label.style())
        self.value.setStyle(self.value.style())

        return ret


    # Return 0 ok, 1 warn, 2 ool
    def update_tm(self, parameter_string, display = True):
        try:
            #mib_id = parameter_string[1]
            #mib_id = parameter_string[iws_defs.TM_MIB_ID_INDEX]
            #label_frame = self.labels[mib_id]

            #raw_val_str = parameter_string[2]
            #eng_val_str = parameter_string[3]
            #status      = parameter_string[4]
            raw_val_str  = parameter_string[iws_defs.TM_RAW_VAL_INDEX] # 2
            eng_val_str  = parameter_string[iws_defs.TM_ENG_VAL_INDEX] # 3
            status       = parameter_string[iws_defs.TM_STATUS_INDEX]  # 4
            eng_val_str1 = parameter_string[iws_defs.TM_ENG_VAL_INDEX + 2] # 3

            raw_val = float(raw_val_str)

#            if fpn.match(eng_val_str):
#                eng_val = float("%3.03f" % float(eng_val_str))
#                eng_val_str = str(eng_val)
#            #else:
#                #eng_val = raw_val

            if self.id == "VOGT2856":
                print(f"{raw_val} {eng_val} {raw_val_str} {eng_val_str} {status}")

            #return self.update_display1(raw_val, eng_val, raw_val_str, eng_val_str, status, display)
            return self.update_display2(raw_val_str, eng_val_str, status, eng_val_str, display)

        except Exception as e:
            print("Exception: LabelFrame:update_tm - " + str(e))
            return 0
            pass
